<?php

declare(strict_types=1);

namespace App\Controller\Entry;

use App\Controller\AbstractController;
use App\DTO\EntryDto;
use App\Entity\Magazine;
use App\PageView\EntryPageView;
use App\Repository\Criteria;
use App\Service\IpResolver;
use App\Service\EntryCommentManager;
use App\Service\EntryManager;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\IsGranted;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Symfony\Component\Validator\Validator\ValidatorInterface;

class EntryCreateController extends AbstractController
{
    use EntryTemplateTrait;
    use EntryFormTrait;

    public function __construct(
        private readonly EntryManager $manager,
        private readonly EntryCommentManager $commentManager,
        private readonly ValidatorInterface $validator,
        private readonly IpResolver $ipResolver
    ) {
    }

    #[IsGranted('ROLE_USER')]
    public function __invoke(?Magazine $magazine, ?string $type, Request $request): Response
    {
        $dto = new EntryDto();
        $dto->magazine = $magazine;

        $form = $this->createFormByType((new EntryPageView(1))->resolveType($type), $dto);
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            $dto = $form->getData();
            $dto->ip = $this->ipResolver->resolve();

            if (!$this->isGranted('create_content', $dto->magazine)) {
                throw new AccessDeniedHttpException();
            }

            $entry = $this->manager->create($dto, $this->getUserOrThrow());

            $this->addFlash(
                'success',
                'flash_thread_new_success'
            );

            return $this->redirectToMagazine(
                $entry->magazine,
                $this->manager->getSortRoute(Criteria::SORT_NEW)
            );
        }

        return $this->render(
            $this->getTemplateName((new EntryPageView(1))->resolveType($type)),
            [
                'magazine' => $magazine,
                'form' => $form->createView(),
            ],
            new Response(null, $form->isSubmitted() && !$form->isValid() ? 422 : 200)
        );
    }
}
